﻿using WingServerCommon.Service;
using WingInterfaceLibrary.Interface;
using System.Threading.Tasks;
using WingInterfaceLibrary.DTO.Message;
using WingInterfaceLibrary.Request.ChatMessage;
using WingInterfaceLibrary.Result;
using WingInterfaceLibrary.Request.DBRequest;
using WingServerCommon.Interfaces.Cache;
using WingInterfaceLibrary.Enum;
using WingInterfaceLibrary.Internal.Request;
using System.Collections.Generic;
using System.Linq;
using WingInterfaceLibrary.DB.Request;
using WingInterfaceLibrary.Enum.CourseEnum;
using WingInterfaceLibrary.DTO.Courses;
using WingInterfaceLibrary.Enum.NotificationEnum;
using WingInterfaceLibrary.Notifications;
using System;
using System.Text.RegularExpressions;

namespace WingUserModule.Service;

public partial class UserService : JsonRpcService, IChatMessageService
{
    /// <summary>
    /// 获取聊天消息分页倒序
    /// </summary>
    /// <param name="request">请求对象</param>
    /// <returns>授权响应</returns>
    /// <remarks>POST</remarks>
    public Task<PageResult<ChatMessagesDTO>> GetChatMessagePagesAsync(GetChatMessagePageRequest request)
    {
        var getChatMessagePagesDBRequest = new GetChatMessagePagesDBRequest();
        getChatMessagePagesDBRequest.CurrentPage = request.PageIndex;
        getChatMessagePagesDBRequest.PageSize = request.PageSize;
        getChatMessagePagesDBRequest.RelevanceCode = request.RelevanceCode;
        var result = _chatMessageDBService.GetChatMessagePagesDBAsync(getChatMessagePagesDBRequest);
        return result;
    }


    /// <summary>
    /// 游客获取聊天消息分页倒序
    /// </summary>
    /// <param name="request">请求对象</param>
    /// <returns>授权响应</returns>
    /// <remarks>POST</remarks>
    public Task<PageResult<ChatMessagesDTO>> GetChatMessagePagesVisitorAsync(GetChatMessagePageVisitorRequest request)
    {
        var getChatMessagePagesDBRequest = new GetChatMessagePagesDBRequest();
        getChatMessagePagesDBRequest.CurrentPage = request.PageIndex;
        getChatMessagePagesDBRequest.PageSize = request.PageSize;
        getChatMessagePagesDBRequest.RelevanceCode = request.RelevanceCode;
        var result = _chatMessageDBService.GetChatMessagePagesDBAsync(getChatMessagePagesDBRequest);
        return result;
    }

    public async Task<ChatMessagesDTO> SendChatMessageAsync(SendChatMessageRequest request)
    {
        var token = request.Token;
        var relevanceCode = request.RelevanceCode;
        var content = request.Content;
        var userCode = await GetClientIdByTokenAsync(token);
        var user = CacheMaintenance.Instance.Get<IUserInfoManager>().Get(userCode);
        var userName = !string.IsNullOrWhiteSpace(user.FullName) ? user.FullName : !string.IsNullOrWhiteSpace(user.NickName) ? user.NickName : user.UserName;
        if(Regex.IsMatch(userName,"[0-9A-Z]{32}") && !string.IsNullOrWhiteSpace(user.Phone))
        {
            userName = user.Phone;
        }
        if(Regex.IsMatch(userName,"[0-9A-Z]{32}") && !string.IsNullOrWhiteSpace(user.Email))
        {
            userName = user.Email;
        }
        var createTime = request.CreateTime > DateTime.MinValue ? request.CreateTime : DateTime.UtcNow;
        var chatMessage = new ChatMessagesDTO
        {
            ChatCmdType = request.ChatCmdType,
            Content = content,
            RelevanceCode = relevanceCode,
            TransactionType = request.TransactionType,
            UserCode = userCode,
            DisplayName = userName,
            HeadImg = user.HeadImageUrl,
            CreateTime = createTime,
        };
        var chatMessageCode = await _chatMessageDBService.AddChatMessageDBAsync(chatMessage);
        if (string.IsNullOrWhiteSpace(chatMessageCode))
        {
            ThrowCustomerException(CustomerRpcCode.SendChatMessageFailed, "Send Chat Message Failed");
        }
        else
        {
            chatMessage.Code = chatMessageCode;
            var course = await _courseDBService.FindCourseByCodeAsync(new FindCourseByCodeDBRequest { Code = relevanceCode });
            if (course != null && course.CourseType == CourseTypeEnum.LiveCourse)
            {
                var clientIds = GetCourseClientIds(course);
                clientIds.RemoveAll(x => x == userCode);
                await BroadcastChatMsgNotification(clientIds, relevanceCode, chatMessage);
            }
        }

        return chatMessage;
    }


    public async Task<ChatMessagesDTO> SendVisitChatMessageAsync(SendVisitChatMessageRequest request)
    {
        var uniqueId = request.Visitor;
        var relevanceCode = request.RelevanceCode;
        var content = request.Content;
        var userName = request.UserName;
        var createTime = request.CreateTime > DateTime.MinValue ? request.CreateTime : DateTime.UtcNow;
        var chatMessage = new ChatMessagesDTO
        {
            ChatCmdType = request.ChatCmdType,
            Content = content,
            RelevanceCode = relevanceCode,
            TransactionType = request.TransactionType,
            Visitor = uniqueId,
            DisplayName = userName,
            CreateTime = createTime
        };
        var chatMessageCode = await _chatMessageDBService.AddChatMessageDBAsync(chatMessage);
        if (string.IsNullOrWhiteSpace(chatMessageCode))
        {
            ThrowCustomerException(CustomerRpcCode.SendChatMessageFailed, "Send Chat Message Failed");
        }
        else
        {
            chatMessage.Code = chatMessageCode;
            var course = await _courseDBService.FindCourseByCodeAsync(new FindCourseByCodeDBRequest { Code = relevanceCode });
            if (course != null && course.CourseType == CourseTypeEnum.LiveCourse)
            {
                var clientIds = GetCourseClientIds(course);
                await BroadcastChatMsgNotification(clientIds, relevanceCode, chatMessage);
            }
        }

        return chatMessage;
    }

    private List<string> GetCourseClientIds(CourseInfoDetailDTO course)
    {
        var clientIds = new List<string>
            {
                course.TeacherCode
            };
        var students = _courseDBService.FindSignInListAsync(new List<string> { course.Code }).Result;
        if (students != null && students.Any())
        {
            foreach (var item in students)
            {
                if (item.SignCourseStatus == StudentCourseStatusEnum.NoSignUp)
                {
                    continue;
                }
                clientIds.Add(item.StudentCode);
            }
        }
        if (course.Assistants != null && course.Assistants.Any())
        {
            foreach (var item in course.Assistants)
            {
                clientIds.Add(item.Code);
            }
        }
        if (course.Experts != null && course.Experts.Any())
        {
            foreach (var item in course.Experts)
            {
                clientIds.Add(item.Code);
            }
        }
        return clientIds.Distinct().ToList();
    }

    private async Task BroadcastChatMsgNotification(List<string> clientIds, string relevanceCode, ChatMessagesDTO chatMessage)
    {
        if (clientIds != null && clientIds.Any())
        {
            var notification = new ChatMsgNotification
            {
                UserCode = chatMessage.UserCode,
                DisplayName = chatMessage.DisplayName,
                HeadImg = chatMessage.HeadImg,
                ChatCmd = chatMessage.ChatCmdType,
                Message = chatMessage.Content,
                SendTime = chatMessage.CreateTime,
            };
            var broadcastNotification = new BroadcastNotificationRequest()
            {
                ClientIds = clientIds,
                Message = notification,
                JsonMessage = Newtonsoft.Json.JsonConvert.SerializeObject(notification),
                NotificationType = NotificationTypeEnum.ChatMsgNotification,
                RelevanceCode = relevanceCode,
                TransactionType = TransactionTypeEnum.Chat,
                ReceiverType = ApplicantTypeEnum.Client,
                WSConnectType = WSConnectTypeEnum.Default,
                IsNeedSyn = false,
            };
            var result = await _notificationService.BroadcastMessageAsync(broadcastNotification);
        }
    }

}
