﻿using System.Net.NetworkInformation;
using System.Linq;
using System.Collections.Generic;
using System.Threading.Tasks;
using WingUserModule.Processor;
using System;
using System.Text;
using JsonRpcLite.Services;
using WingInterfaceLibrary.Interface;
using WingInterfaceLibrary.Request.Authentication;
using WingUserModule.Entity;
using System.Text.RegularExpressions;
using WingInterfaceLibrary.Request.User;
using WingInterfaceLibrary.Enum;
using WingInterfaceLibrary.DTO.User;
using WingServerCommon.Service;
using WingServerCommon.Mapper;
using WingInterfaceLibrary.DB.Request;
using WingInterfaceLibrary.Request;
using WingUserModule.Helper;
using WingServerCommon.Log;
using WingServerCommon.Utilities;
using WingServerCommon.Interfaces.PasswordVerify;
using WingServerCommon.Interfaces.Cache;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using System.Net.Http;
using WingInterfaceLibrary.Request.Storage;
using WingServerCommon.Interfaces.FileTransfer;
using System.IO;
using System.Net;
using System.Net.Http.Headers;
using WingUserModule.Common;
using WingServerCommon.Config;
using WingInterfaceLibrary.Internal.Request;
using WingInterfaceLibrary.Result.User;
using WingInterfaceLibrary.Internal.Interface;
using WingServerCommon.Config.Parameters;
using System.Security.Cryptography;

namespace WingUserModule.Service
{
    public partial class UserService : JsonRpcService, ILoginService, ISyncLoginService
    {
        private string _dongleEncryptKey = "VLab_{0}_IDS";
        private string _defaultLicenseId = "3ce681e1-e884-492e-8001-73d488239b08";

        /// <summary>
        /// 检查短信验证码
        /// </summary>
        /// <param name="userPhone">用户手机号</param>
        /// <param name="verifyCode">验证码</param>
        /// <returns>是否成功</returns>
        public async Task<bool> CheckSMSVerificationCodeAsync(CheckSMSVerificationCodeRequest request)
        {
            VerifyPhoneValue(request.UserPhone, request.LanguageCode);
            return await _smsService.CheckVerificationCodeAsync(request.UserPhone, request.VerifyCode);
        }

        /// <summary>
        /// 发送短信验证码
        /// </summary>
        /// <param name="userPhone">用户手机号</param>
        /// <returns>是否成功</returns>
        public async Task<bool> SendSMSVerificationCodeAsync(SendSMSVerificationCodeRequest request)
        {
            VerifyPhoneValue(request.UserPhone, request.LanguageCode);
            var smsTemplateId = "1009241";
            return await _smsService.SendMessageAsync(new List<string> { request.UserPhone }, smsTemplateId);
        }

        /// <summary>
        /// 发送邮箱验证码
        /// </summary>
        /// <param name="emailAddress">邮箱地址</param>
        /// <returns>是否成功</returns>
        public async Task<bool> SendEmailVerificationCodeAsync(SendEmailVerificationCodeRequest request)
        {
            if (string.IsNullOrWhiteSpace(request.EmailAddress) || !Regex.IsMatch(request.EmailAddress, _RegexEmail))
            {
                ThrowCustomerException(CustomerRpcCode.EmailIncorrect, "Email formal error!");
            }
            var verifyCode = GetVerifyCode(request.EmailAddress, 5);
            return await _emailService.SendEmailAsync(request.LanguageCode, new List<string> { request.EmailAddress }, "Simple", verifyCode);
        }

        /// <summary>
        /// 检查邮箱验证码
        /// </summary>
        /// <param name="emailAddress">邮箱地址</param>
        /// <param name="verifyCode">验证码</param>
        /// <returns>是否成功</returns>
        public async Task<bool> CheckEmailVerificationCodeAsync(CheckEmailVerificationCodeRequest request)
        {
            VerifyVerificationCodeValue(request.VerifyCode);
            if (string.IsNullOrWhiteSpace(request.EmailAddress) || !Regex.IsMatch(request.EmailAddress, _RegexEmail))
            {
                ThrowCustomerException(CustomerRpcCode.EmailIncorrect, "Email formal error!");
                return false;
            }
            if (_emailVerifyCodes.ContainsKey(request.EmailAddress))
            {
                var verifyInfo = _emailVerifyCodes[request.EmailAddress];
                if (verifyInfo.VerifyCode == request.VerifyCode)
                {
                    var result = !_emailVerifyCodes[request.EmailAddress].IsOverdue;
                    if (result)
                    {
                        return await Task.FromResult(result);
                    }
                    else
                    {
                        ThrowCustomerException(CustomerRpcCode.SMSVerifyCodeExpired, "Verification code expired");
                        return false;
                    }
                }
            }
            ThrowCustomerException(CustomerRpcCode.VerificationCodeIllegality, "VerificationCode is illegality");
            return false;
        }

        /// <summary>
        /// 获取邮箱Base64字符串
        /// </summary>
        /// <param name="content"></param>
        /// <returns></returns>
        private string GetBase64EmailString(string content)
        {
            content = Convert.ToBase64String(Encoding.UTF8.GetBytes(content));
            var str = $"=?UTF8?B?{content}?=";
            return str;
        }

        /// <summary>
        /// 获取邮箱验证码（私有）
        /// </summary>
        /// <param name="emailAddress">邮箱地址</param>
        /// <param name="validMinutes">查找时间范围：如5分钟之前</param>
        /// <returns></returns>
        private string GetVerifyCode(string emailAddress, int validMinutes)
        {
            if (_emailVerifyCodes.ContainsKey(emailAddress))
            {
                _emailVerifyCodes.TryRemove(emailAddress, out _);
            }
            var mobileVerifyCode = _emailVerifyCodes.GetOrAdd(emailAddress, new VerifyCodeInfo
            {
                VerifyCode = new Random().Next(1000, 9999).ToString(),
                ValidMinutes = validMinutes,
                CreateTime = DateTime.UtcNow,
            });
            return mobileVerifyCode.VerifyCode;
        }

        /// <summary>
        /// 根据手机找回密码
        /// </summary>
        /// <param name="phone">手机号</param>
        /// <param name="verifyCode">验证码</param>
        /// <param name="newPassword">新密码</param>
        /// <returns>是否成功</returns>
        public async Task<bool> RetrievePasswordByPhoneAsync(RetrievePasswordByPhoneRequest request)
        {
            var userInfo = await _userDBService.FindUserByPhoneAsync(request.Phone);
            if (userInfo != null)
            {
                var isVerify = await _smsService.CheckVerificationCodeAsync(request.Phone, request.VerifyCode);
                if (isVerify)
                {
                    CacheMaintenance.Instance.Get<IUserInfoManager>().Remove(userInfo.UserCode);
                    return await _userDBService.UpdateUserPasswordAsync(userInfo.UserCode, request.NewPassword.ToString());
                }
                return false;
            }
            else
            {
                ThrowCustomerException(CustomerRpcCode.UserNotRegistered, "User not registered");
                return false;
            }
        }

        /// <summary>
        /// 根据邮箱找回密码
        /// </summary>
        /// <param name="mail">邮箱地址</param>
        /// <param name="verifyCode">验证码</param>
        /// <param name="newPassword">新密码</param>
        /// <returns>是否成功</returns>
        public async Task<bool> RetrievePasswordByEmailAsync(RetrievePasswordByEmailRequest request)
        {
            if (string.IsNullOrWhiteSpace(request.Mail) || !Regex.IsMatch(request.Mail, _RegexEmail))
            {
                ThrowCustomerException(CustomerRpcCode.EmailIncorrect, "Email formal error!");
            }
            var userInfo = await _userDBService.FindUserByEmailAsync(request.Mail);
            if (userInfo != null)
            {
                if (_emailVerifyCodes.ContainsKey(request.Mail))
                {
                    var isVerify = !_emailVerifyCodes[request.Mail].IsOverdue && _emailVerifyCodes[request.Mail].VerifyCode == request.VerifyCode;
                    if (isVerify)
                    {
                        //密码后期需要加密
                        CacheMaintenance.Instance.Get<IUserInfoManager>().Remove(userInfo.UserCode);
                        return await _userDBService.UpdateUserPasswordAsync(userInfo.UserCode, request.NewPassword.ToString());
                    }
                    else
                    {
                        ThrowCustomerException(CustomerRpcCode.VerificationCodeIllegality, "VerificationCode is illegality");
                        return false;
                    }
                }
                else
                {
                    ThrowCustomerException(CustomerRpcCode.VerificationCodeIllegality, "VerificationCode is illegality");
                    return false;
                }
            }
            else
            {
                ThrowCustomerException(CustomerRpcCode.UserNotRegistered, "User not registered");
                return false;
            }
        }

        /// <summary>
        /// 检查登录类型
        /// </summary>
        /// <param name="anyAccount">登录账号</param>
        /// <returns>登录类型枚举值</returns>
        public async Task<int> CheckLoginTypeAsync(CheckLoginTypeRequest request)
        {
            return await CheckLoginTypeWithExcept(request);
        }

        /// <summary>
        /// 退出登录
        /// </summary>
        /// <param name="request">登录token信息</param>
        /// <returns>是否成功</returns>
        public async Task<bool> LoginOutAsync(TokenRequest request)
        {
            var userCode = await GetClientIdByTokenAsync(request.Token);
            if (string.IsNullOrEmpty(userCode))
            {
                return true;
            }
            //查询所有该老师的正在直播的课程
            var dbRequest = new FindCoursePagesDBRequest()
            {
                CourseType = WingInterfaceLibrary.Enum.CourseEnum.CourseTypeEnum.LiveCourse,
                Status = WingInterfaceLibrary.Enum.CourseEnum.CourseStatusEnum.Started,
                PageIndex = 1,
                PageSize = 100,
                TeacherCode = userCode
            };
            var dataList = _courseDBService.FindCoursePagesAsync(dbRequest).Result;
            if (dataList?.PageData?.Count > 0)
            {
                foreach (var item in dataList.PageData)
                {
                    var courseReq = new WingInterfaceLibrary.Request.Education.LeaveLiveCourseRequest()
                    {
                        Token = request.Token,
                        CourseCode = item.Code
                    };
                    await _educationService.LeaveLiveCourseAsync(courseReq);
                }
            }
            var loginOutResult = await _authenticationService.LogOffAsync(request);
            if (loginOutResult)
            {
                //移除websocket
                var removeWebSocketResult = await _notificationService.RemoveToken(request);
                loginOutResult = loginOutResult & removeWebSocketResult;
            }
            return loginOutResult;
        }

        /// <summary>
        /// 踢出功能
        /// </summary>
        /// <param name="request">登录token信息</param>
        /// <returns>是否成功</returns>
        public async Task<bool> LoginOutByAccountAsync(CommonLoginRequest request)
        {
            var user = await _userDBService.FindUserByNameAsync(request.AnyAccount);
            var getTokenRequest = new GetTokensWithClientIdRequest { ClientId = user.UserCode };
            var tokens = await _authenticationService.GetTokensWithClientIdAsync(getTokenRequest);
            var token = tokens.FirstOrDefault(x => x.Platform == request.Platform);
            var logOffRequest = new TokenRequest { Token = token.Code };
            var loginOutResult = await _authenticationService.LogOffAsync(logOffRequest);
            return loginOutResult;
        }

        /// <summary>
        /// 通用登录
        /// </summary>
        /// <param name="anyAccount">用户名/手机/邮箱</param>
        /// <param name="anyCode">密码/验证码</param>
        /// <param name="password">密码</param>
        /// <returns>Token信息</returns>
        public async Task<LoginResult> CommonLoginAsync(CommonLoginRequest request)
        {
            try
            {
                if (!string.IsNullOrWhiteSpace(request.LabDongle))
                {
                    return await LoginByLabDongleAsync(request);
                }
                var loginAccount = request.AnyAccount;
                //VerifyAccountValue(request.AnyAccount);//老数据迁移存在一个汉字的账号此处去除账号校验
                if (string.IsNullOrWhiteSpace(request.AnyAccount))
                {
                    ThrowCustomerException(CustomerRpcCode.AccountIsEmpty, "Account is empty");
                }
                var userLoginInfo = new UserLoginInfo();
                ILoginProcessor processor = null;
                var loginType = await CheckLoginTypeAsync(new CheckLoginTypeRequest { AnyAccount = request.AnyAccount });
                var enumLoginType = (LoginProcessorTypeEnum)loginType;
                if (enumLoginType != LoginProcessorTypeEnum.Unregistered)
                {
                    if (!string.IsNullOrWhiteSpace(request.AnyCode))
                    {
                        //手机号
                        if (Regex.IsMatch(request.AnyAccount, "^1[0-9]{10}$"))
                        {
                            enumLoginType = LoginProcessorTypeEnum.Phone;
                        }
                        else
                        {
                            enumLoginType = LoginProcessorTypeEnum.Email;
                        }
                    }
                }
                if (string.IsNullOrWhiteSpace(request.Password) && !string.IsNullOrWhiteSpace(request.AnyCode))
                {
                    //如未注册就先注册
                    if (enumLoginType == LoginProcessorTypeEnum.Unregistered)
                    {
                        await CommonSignUpAsync(new CommonSignUpRequest { AnyAccount = request.AnyAccount, AnyCode = request.AnyCode, Password = "Symmetry_mk9xah8N" });//abc123
                        loginType = await CheckLoginTypeAsync(new CheckLoginTypeRequest { AnyAccount = request.AnyAccount });
                        enumLoginType = (LoginProcessorTypeEnum)loginType;
                    }
                    processor = LoginProcessorCreator.GetProcessor(enumLoginType);
                    userLoginInfo.LoginType = enumLoginType;
                    switch (userLoginInfo.LoginType)
                    {
                        case LoginProcessorTypeEnum.Phone:
                            var checkPhoneRight = await _smsService.CheckVerificationCodeAsync(request.AnyAccount, request.AnyCode);
                            if (checkPhoneRight)
                            {
                                userLoginInfo.Phone = request.AnyAccount.ToString();
                            }
                            else
                            {
                                ThrowCustomerException(CustomerRpcCode.VerificationCodeIllegality, "VerificationCode is illegality");
                            }
                            break;
                        case LoginProcessorTypeEnum.Email:
                            var checkEmailRight = await CheckEmailVerificationCodeAsync(new CheckEmailVerificationCodeRequest { EmailAddress = request.AnyAccount.ToString(), VerifyCode = request.AnyCode });
                            if (checkEmailRight)
                            {
                                userLoginInfo.Email = request.AnyAccount.ToString();
                            }
                            break;
                    }
                }
                else if (!string.IsNullOrWhiteSpace(request.Password) && string.IsNullOrWhiteSpace(request.AnyCode))
                {
                    if (enumLoginType == LoginProcessorTypeEnum.Unregistered)
                    {
                        ThrowCustomerException(CustomerRpcCode.UserNotRegistered, "User not registered");
                    }
                    processor = LoginProcessorCreator.GetProcessor(LoginProcessorTypeEnum.Official);
                    switch (enumLoginType)
                    {
                        case LoginProcessorTypeEnum.Official:
                            userLoginInfo.UserName = request.AnyAccount;
                            userLoginInfo.Password = request.Password;
                            break;
                        case LoginProcessorTypeEnum.Phone:
                            userLoginInfo.Phone = request.AnyAccount.ToString();
                            VerifyPhoneValue(userLoginInfo.Phone, request.LanguageCode);
                            userLoginInfo.Password = request.Password;
                            break;
                        case LoginProcessorTypeEnum.Email:
                            userLoginInfo.Email = request.AnyAccount.ToString();
                            VerifyEmailValue(userLoginInfo.Email);
                            userLoginInfo.Password = request.Password;
                            break;
                    }
                }
                else
                {
                    ThrowCustomerException(CustomerRpcCode.IncorrectPasswordFormat, "Incorrect password format");
                }
                userLoginInfo.Platform = request.Platform;
                userLoginInfo.LoginSource = request.LoginSource;
                string token = string.Empty;
                try
                {
                    userLoginInfo.InstallVersion = request.InstallVersion;
                    token = await processor.DoLoginAsync(userLoginInfo);
                    ///解决验证码登录没有用户编码的问题
                    if (!string.IsNullOrWhiteSpace(token))
                    {
                        var userCode = await GetClientIdByTokenAsync(token);
                        if (!string.IsNullOrWhiteSpace(userCode))
                        {
                            CacheMaintenance.Instance.Get<IUserInfoManager>().Remove(userCode);
                        }
                    }
                }
                catch (RpcException ex)
                {
                    if (ex.ErrorCode == (int)CustomerRpcCode.PasswordError)
                    {
                        if (!string.IsNullOrWhiteSpace(userLoginInfo.UserCode))
                        {
                            var passwordError = GetGradingProtectionConfigCache("PasswordErrorLimit01");
                            if (passwordError)
                            {
                                var loginLockInfo = _userLoginLock.CheckUserLoginLocked(userLoginInfo.UserCode);
                                if (!loginLockInfo.IsLocked)
                                {
                                    return new LoginResult()
                                    {
                                        LoginState = LoginStateEnum.PasswordIncorrect,
                                        LockRemainingTimes = loginLockInfo?.RemainingTimes
                                    };
                                }
                                ThrowCustomerException(CustomerRpcCode.AccountIsLocked, "Account is locked");
                            }
                        }
                    }
                    else if (ex.ErrorCode == (int)CustomerRpcCode.AccountIsLocked)
                    {
                        ThrowCustomerException(CustomerRpcCode.AccountIsLocked, "Account is locked");
                    }
                    throw;
                }
                if (!string.IsNullOrWhiteSpace(token))
                {
                    _userHeartRateManager.AddOrUpdate(token);
                    var refreshRequest = new RefreshStaticticRecordsRequest { UserCode = userLoginInfo.UserCode, InExecutor = true };
                    RefreshStaticticRecordsAsync(refreshRequest);
                }
                var passwordExpired = await VerifyUserPasswordExpire(token);
                return new LoginResult()
                {
                    Token = token,
                    LoginState = LoginStateEnum.Succeed,
                    PasswordExpired = passwordExpired,
                    AccountName = request.AnyAccount,
                };
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"common login failed,{request.AnyAccount}, ex:{ex}");
                throw;
            }
        }

        /// <summary>
        /// 科研版Dongle登录/注册
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        private async Task<LoginResult> LoginByLabDongleAsync(CommonLoginRequest request)
        {
            var anyCode = request.AnyCode;// Dongle密文
            var labDongle = request.LabDongle;
            var plateform = request.Platform;
            var loginSource = request.LoginSource;
            var licenseId = request.AnyAccount;
            var loginServer = ConfigurationManager.Host;
            var ipValue = 0;

            Logger.WriteLineInfo($"UserService LoginByLabDongleAsync, dongle:{labDongle}, plateform:{plateform.ToString()}, loginSource:{loginSource.ToString()}");
            var userCode = string.Empty;
            if (!string.IsNullOrWhiteSpace(labDongle))
            {
                // dongle和盐加密后验证dongle的合法性
                if (ToSHA256(string.Format(_dongleEncryptKey, labDongle)) != anyCode)
                {
                    ThrowCustomerException(CustomerRpcCode.DongleMismatch, "Dongle Mismatch");
                }
                var serverLicenseInfo = GetLicenseId();
                if (serverLicenseInfo.Item1)
                {
                    if (!string.IsNullOrWhiteSpace(serverLicenseInfo.Item2))
                    {
                        if (licenseId != serverLicenseInfo.Item2)
                        {
                            ThrowCustomerException(CustomerRpcCode.LicenseMismatch, "License Mismatch");
                        }
                    }
                    else
                    {
                        ThrowCustomerException(CustomerRpcCode.LicenseMismatch, "License Mismatch");
                    }
                }
                var user = await _userDBService.FindUserByLabDongleDBAsync(labDongle);
                var userName = user?.UserName ?? string.Empty;
                if (user == null || string.IsNullOrWhiteSpace(user.LabDongle) || string.IsNullOrWhiteSpace(user.UserCode))
                {
                    ThrowCustomerException(CustomerRpcCode.UserNotRegistered, "User not registered");
                    userName = Guid.NewGuid().ToString("N").ToUpper();
                    var password = "Symmetry_mk9xah8N";// abc123

                    var userPasswordDTO = new UserPasswordDTO
                    {
                        Phone = string.Empty,
                        Email = string.Empty,
                        UserName = userName,
                        LabDongle = labDongle,
                        SecretPassword = password,
                        PasswordUpdateTime = DateTime.UtcNow,
                        PasswordRecords = new List<string>() { password },
                        RoleCodes = new List<string> { "Role_GeneralUser" }
                    };
                    var allFeaturesResult = await _userFeaturesDBService.FindUserFeaturePagesAsync(new PageFilterRequest { CurrentPage = 1, PageSize = 9999, Filter = new Dictionary<string, string>(), IsFuzzy = true });
                    if (allFeaturesResult?.TotalCount > 0)
                    {
                        var findDataList = allFeaturesResult.PageData.Where(c => c.IsResearchProjectUser == true)?.ToList();
                        if (findDataList?.Count > 0)
                        {
                            userPasswordDTO.UserFeatureCodes = findDataList.Select(f => f.FeatureCode).Distinct().ToList();
                        }
                    }

                    userCode = await _userDBService.InsertUserAsync(new CreateUserInfoDBRequest
                    {
                        Data = userPasswordDTO,
                        PlatformDatas = new Dictionary<string, string>(),
                        ExtensionData = string.Empty,
                    });
                    if (string.IsNullOrWhiteSpace(userCode))
                    {
                        return new LoginResult()
                        {
                            Token = string.Empty,
                            LoginState = LoginStateEnum.SignOrLoginFail,
                            PasswordExpired = false,
                            AccountName = string.Empty,
                        };
                    }
                }
                else
                {
                    userCode = user.UserCode;
                }
                var tokenInfo = await _authenticationService.ApplyTokenAsync(new ApplyTokenRequest(AccountType.User, labDongle, plateform, loginSource, userCode, loginServer, ipValue, ""));

                if (!string.IsNullOrWhiteSpace(tokenInfo?.Code))
                {
                    Logger.WriteLineInfo($"UserService LoginByLabDongleAsync success, dongle:{labDongle}, anyAccount:{request.AnyAccount}. licenseId:{licenseId}, userCode:{userCode},, token:{tokenInfo.Code}");
                    return new LoginResult()
                    {
                        Token = tokenInfo.Code,
                        LoginState = LoginStateEnum.Succeed,
                        PasswordExpired = false,
                        AccountName = userName,
                        LicenseId = licenseId,
                    };
                }
            }
            return new LoginResult()
            {
                Token = string.Empty,
                LoginState = LoginStateEnum.SignOrLoginFail,
                PasswordExpired = false,
                AccountName = string.Empty,
            };
        }

        private Tuple<bool, string> GetLicenseId()
        {
            var valid = false;
            var licenseId = "";
            try
            {
                valid = !ConfigurationManager.GetParammeter<BoolParameter>("General", "NotValidateLicense").Value;
                if (valid)
                {
                    licenseId = EnvironmentConfigManager.GetParammeter<StringParameter>("Services", "SoftwareLicenseId").Value;
                }
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"UserService GetLicenseId failed, ex:{ex}");
            }
            return new Tuple<bool, string>(valid, licenseId);
        }

        public string ToSHA256(string originalString)
        {
            using (SHA256 sha256Hash = SHA256.Create())
            {
                byte[] bytes = sha256Hash.ComputeHash(Encoding.UTF8.GetBytes(originalString));

                StringBuilder builder = new StringBuilder();
                for (int i = 0; i < bytes.Length; i++)
                {
                    builder.Append(bytes[i].ToString("x2"));
                }

                string encryptedString = builder.ToString().Substring(0, 32); // 获取前32位作为加密后的字符串
                return encryptedString;
            }
        }

        /// <summary>
        /// 通用用户注销
        /// </summary>
        /// <param name="request">注销请求实体</param>
        /// <returns>是否成功</returns>
        public async Task<bool> CommonUserLogOffAsync(CommonLogOffRequest request)
        {
            var userCode = await GetClientIdByTokenAsync(request.Token);
            var userInfo = await _userDBService.FindUserPasswordByCodeAsync(userCode);
            if (userInfo == null)
            {
                ThrowCustomerException(CustomerRpcCode.UserNotExisted, "User not found");
            }
            // if (request.VerificationType == VerificationTypeEnum.PhoneVerificationCode)
            // {
            //     var checkPhoneRight = await _smsService.CheckVerificationCodeAsync(userInfo.Phone, request.AnyCode);
            //     if (!checkPhoneRight)
            //     {
            //         ThrowCustomerException(CustomerRpcCode.VerificationCodeIllegality, "VerificationCode is illegality");
            //     }
            // }
            // else if (request.VerificationType == VerificationTypeEnum.EmailVerificationCode)
            // {
            //     var emailRequest = new CheckEmailVerificationCodeRequest { 
            //         EmailAddress = userInfo.Email, 
            //         VerifyCode = request.AnyCode 
            //     };
            //     var checkEmailRight = await CheckEmailVerificationCodeAsync(emailRequest);
            //     if (!checkEmailRight)
            //     {
            //         ThrowCustomerException(CustomerRpcCode.VerificationCodeIllegality, "VerificationCode is illegality");
            //     }
            // }
            // else if (request.VerificationType == VerificationTypeEnum.Password)
            // {
            //     var checkPasswordRight = await _userDBService.VerificationPasswordAsync(userInfo.UserCode, request.AnyCode);
            //     if (!checkPasswordRight)
            //     {
            //         throw new RpcException((int)CustomerRpcCode.PasswordError, "Password is wrong!", "Password is wrong!");
            //     }
            // }
            //清理User关联信息
            var res = await _userDBService.CommonUserLogOffDBAsync(userInfo);
            return res;
        }

        /// <summary>
        /// 微信通用注册
        /// </summary>
        /// <param name="anyAccount">用户名/手机/邮箱</param>
        /// <param name="anyCode">验证码</param>
        /// <param name="password">密码</param>
        /// <returns>是否成功</returns>
        public async Task<LoginResult> WXCommonSignUpAsync(CommonSignUpRequest request)
        {
            var appId = EnvironmentConfigManager.GetParammeter<StringParameter>("WXOpenApi", "IOTAppKey").Value;
            var appSecret = EnvironmentConfigManager.GetParammeter<StringParameter>("WXOpenApi", "IOTAppSecret").Value;
            if (string.IsNullOrEmpty(request.OpenId))
            {
                request.OpenId = await GetWXOpenId(appId, appSecret, request.AnyCode);
            }
            if (!string.IsNullOrEmpty(request.OpenId))
            {
                //根据openid验证用户是否注册过
                var userInfo = await _userDBService.FindUserByOpenIdAsync(request.OpenId);
                if (!string.IsNullOrEmpty(userInfo?.UserCode))
                {
                    var userPasswordInfo = await _userDBService.FindUserPasswordByCodeAsync(userInfo.UserCode);
                    //找到了。表示注册过，直接登录；
                    var loginReq = new CommonLoginRequest()
                    {
                        AnyAccount = userInfo.UserName,
                        Password = userPasswordInfo.SecretPassword,
                        Platform = request.Platform,
                        LoginSource = LoginSource.WX
                    };
                    var res = await CommonLoginAsync(loginReq);
                    return res;
                }
            }
            if (string.IsNullOrEmpty(request.WXPhoneAnyCode))
            {
                // ThrowCustomerException(CustomerRpcCode.UserNotRegistered, "User not registered");
                return new LoginResult()
                {
                    OpenId = request.OpenId,
                    LoginState = LoginStateEnum.SignOrLoginFail
                };
            }
            var accessToken = await GetWXGetAccessToken(appId, appSecret);
            request.AnyAccount = await GetWXPhoneNumberByWXCode(accessToken, request.WXPhoneAnyCode);
            if (string.IsNullOrEmpty(request.AnyAccount))
            {
                ThrowCustomerException(CustomerRpcCode.PhoneEmptyError, "Phone empty error");
                return new LoginResult();
            }
            VerifyAccountValue(request.AnyAccount);
            var loginType = await CheckLoginTypeAsync(new CheckLoginTypeRequest { AnyAccount = request.AnyAccount });
            try
            {
                var loginTypeEnum = (LoginProcessorTypeEnum)loginType;
                switch (loginTypeEnum)
                {
                    case LoginProcessorTypeEnum.Phone:
                        ThrowCustomerException(CustomerRpcCode.PhoneRegistered, "Phone is registered");
                        return new LoginResult();
                }
                string phone = string.Empty, userName = request.AnyAccount;
                if (Regex.IsMatch(request.AnyAccount, "^1[0-9]{10}$"))
                {
                    phone = request.AnyAccount;
                    userName = CodeCreator.CreateCode("User");
                }
                var password = "Symmetry_mk9xah8N";
                var userPasswordDTO = new UserPasswordDTO
                {
                    Phone = phone,
                    Email = "",
                    UserName = userName,
                    SecretPassword = password,
                    PasswordUpdateTime = DateTime.UtcNow,
                    PasswordRecords = new List<string>() { password },
                    RoleCodes = new List<string> { "Role_GeneralUser" },
                    OpenId = request.OpenId
                };
                var result = await _userDBService.InsertUserAsync(new CreateUserInfoDBRequest
                {
                    Data = userPasswordDTO,
                    PlatformDatas = request.HeaderMap,
                    ExtensionData = string.Empty
                });
                if (!string.IsNullOrEmpty(result))
                {
                    var loginReq = new CommonLoginRequest()
                    {
                        AnyAccount = request.AnyAccount,
                        Password = password,
                        Platform = request.Platform,
                        LoginSource = LoginSource.WX
                    };
                    var res = await CommonLoginAsync(loginReq);
                    return res;
                }
                return new LoginResult()
                {
                    OpenId = request.OpenId,
                    LoginState = LoginStateEnum.SignOrLoginFail
                };
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"wx common sign up failed, ex:{ex}");
                throw;
            }
        }

        #region wx 小程序
        /// <summary>
        /// 获取OpenId
        /// </summary>
        /// <param name="accessToken">令牌</param>
        /// <param name="code">临时code</param>
        /// <returns></returns>
        private async Task<string> GetWXOpenId(string appId, string appSecret, string code)
        {
            var url = $"https://api.weixin.qq.com/sns/jscode2session?appid={appId}&secret={appSecret}&js_code={code}&grant_type=authorization_code";
            var sessionJson = await SendWXRequest(url, "", HttpMethod.Get);
            string openId = "";
            if (!string.IsNullOrEmpty(sessionJson))
            {
                JObject json = JObject.Parse(sessionJson);
                openId = json["openid"].ToString();
            }
            return openId;
        }

        /// <summary>
        /// 获取手机号
        /// </summary>
        /// <param name="accessToken">令牌</param>
        /// <param name="code">临时code</param>
        /// <returns></returns>
        private async Task<string> GetWXPhoneNumberByWXCode(string accessToken, string code)
        {
            var url = $"https://api.weixin.qq.com/wxa/business/getuserphonenumber?access_token={accessToken}";
            var reqJson = @"{
                ""code"": """ + code + @"""
            }";
            var json = await SendWXRequest(url, reqJson, HttpMethod.Post);
            string phoneNumber = "";
            if (!string.IsNullOrEmpty(json))
            {
                JObject jsonObj = JObject.Parse(json);
                var errorCode = jsonObj["errcode"].ToString();
                if (errorCode == "0")
                {
                    phoneNumber = jsonObj["phone_info"]["purePhoneNumber"].ToString();//无区号的手机号
                }
            }
            return phoneNumber;
        }

        /// <summary>
        /// 获取AccessToken
        /// </summary>
        /// <param name="appId">小程序唯一凭证，即 AppID</param>
        /// <param name="appSecret">小程序唯一凭证密钥，即 AppSecret</param>
        /// <returns></returns>
        private async Task<string> GetWXGetAccessToken(string appId, string appSecret)
        {
            var url = $"https://api.weixin.qq.com/cgi-bin/token?grant_type=client_credential&appid={appId}&secret={appSecret}";
            var accessTokenJson = await SendWXRequest(url, "", HttpMethod.Get);
            string accessToken = "";
            if (!string.IsNullOrEmpty(accessTokenJson))
            {
                JObject json = JObject.Parse(accessTokenJson);
                accessToken = json["access_token"].ToString();
            }
            return accessToken;
        }

        /// <summary>
        /// 发送请求
        /// </summary>
        /// <param name="url">服务器</param>
        /// <param name="req">请求信息</param>
        /// <returns></returns>
        private async Task<string> SendWXRequest(string url, string req, HttpMethod method)
        {
            var request = new HttpRequestMessage(method, url);
            request.Content = new StringContent(req);
            var result = _httpClient.Send(request);
            return await result.Content.ReadAsStringAsync();
        }
        #endregion 

        /// <summary>
        /// 通用注册
        /// </summary>
        /// <param name="anyAccount">用户名/手机/邮箱</param>
        /// <param name="anyCode">验证码</param>
        /// <param name="password">密码</param>
        /// <returns>是否成功</returns>
        public async Task<bool> CommonSignUpAsync(CommonSignUpRequest request)
        {
            VerifyAccountValue(request.AnyAccount);
            var loginType = await CheckLoginTypeAsync(new CheckLoginTypeRequest { AnyAccount = request.AnyAccount });
            try
            {
                var loginTypeEnum = (LoginProcessorTypeEnum)loginType;
                switch (loginTypeEnum)
                {
                    case LoginProcessorTypeEnum.Official:
                        ThrowCustomerException(CustomerRpcCode.UserNameRegistered, "UserName is registered");
                        return false;
                    case LoginProcessorTypeEnum.Phone:
                        ThrowCustomerException(CustomerRpcCode.PhoneRegistered, "Phone is registered");
                        return false;
                    case LoginProcessorTypeEnum.Email:
                        ThrowCustomerException(CustomerRpcCode.EmailRegistered, "Email is registered");
                        return false;
                        // case LoginProcessorTypeEnum.Wechat:
                        //     userLoginInfo.ProcessorInfo = new LoginProcessorInfo { UserName = anyAccount, Password = anyCode }
                        //     break;
                }
                string phone = string.Empty, userName = request.AnyAccount, emailAddress = string.Empty;
                if (Regex.IsMatch(request.AnyAccount, "^1[0-9]{10}$"))
                {
                    if (!await CheckSMSVerificationCodeAsync(new CheckSMSVerificationCodeRequest { UserPhone = request.AnyAccount, VerifyCode = request.AnyCode, LanguageCode = request.LanguageCode }))
                    {
                        ThrowCustomerException(CustomerRpcCode.VerificationCodeIllegality, "VerificationCode is illegality");
                        return false;
                    }
                    phone = request.AnyAccount;
                    userName = CodeCreator.CreateCode("User");
                }
                else if (Regex.IsMatch(request.AnyAccount, "^.+@[a-zA-Z0-9.]+$"))
                {
                    if (!await CheckEmailVerificationCodeAsync(new CheckEmailVerificationCodeRequest { EmailAddress = request.AnyAccount.ToString(), VerifyCode = request.AnyCode }))
                    {
                        ThrowCustomerException(CustomerRpcCode.VerificationCodeIllegality, "VerificationCode is illegality");
                        return false;
                    }
                    emailAddress = request.AnyAccount;
                    userName = CodeCreator.CreateCode("User");
                }
                else
                {
                    return await Task.FromResult(false);
                }
                //if (string.IsNullOrWhiteSpace(request.Password) || !Regex.IsMatch(request.Password, @"^(?=.*[A-Za-z])(?=.*\d)[A-Za-z\d]{6,12}$"))
                if (string.IsNullOrWhiteSpace(request.Password))
                {
                    ThrowCustomerException(CustomerRpcCode.NewPasswordEmpty, "Incorrect password format");
                    return false;
                }
                var userPasswordDTO = new UserPasswordDTO
                {
                    Phone = phone,
                    Email = emailAddress,
                    UserName = userName,
                    SecretPassword = request.Password,
                    PasswordUpdateTime = DateTime.UtcNow,
                    PasswordRecords = new List<string>() { request.Password },
                    RoleCodes = new List<string> { "Role_GeneralUser" }
                };
                // var userRoleInfo = await _roleDBService.FindRoleByCodeAsync("Role_GeneralUser");
                // if (!string.IsNullOrEmpty(userRoleInfo?.UserGroupCode)) 
                // {
                //     var userAuthInfo = await _userAuthorityGroupsDBService.FindUserAuthorityGroupByCodeAsync(userRoleInfo.UserGroupCode);
                //     if (userAuthInfo?.Features?.Count > 0) 
                //     {
                //         userPasswordDTO.UserFeatureCodes = userAuthInfo.Features;
                //     }
                // }
                await _userDBService.InsertUserAsync(new CreateUserInfoDBRequest
                {
                    Data = userPasswordDTO,
                    PlatformDatas = request.HeaderMap,
                    ExtensionData = string.Empty
                });
                return await Task.FromResult(true);
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"common sign up failed, ex:{ex}");
                throw;
            }
        }

        public async Task<bool> VerifyAccountAsync(VerifyAccountRequest request)
        {
            VerifyAccountValue(request.UserName);
            int type = 0;
            if (!string.IsNullOrWhiteSpace(request.UserName))
            {
                if (Regex.IsMatch(request.UserName, _RegexEmail))
                {
                    type = await CheckLoginTypeAsync(new CheckLoginTypeRequest { AnyAccount = request.UserName });
                }
                else if (Regex.IsMatch(request.UserName, _RegexPhone))
                {
                    type = await CheckLoginTypeAsync(new CheckLoginTypeRequest { AnyAccount = request.UserName });
                }
                else if (Regex.IsMatch(request.UserName, _RegexUserName))
                {
                    type = await CheckLoginTypeAsync(new CheckLoginTypeRequest { AnyAccount = request.UserName });
                }
                if (type == (int)LoginProcessorTypeEnum.Unregistered)
                {
                    return true;
                }
            }
            return false;
        }

        private async Task<int> CheckLoginTypeWithExcept(CheckLoginTypeRequest request, List<string> exceptUserCodes = null)
        {
            var checkUser = (UserDTO u, List<string> e) => u != null && (e == null || (!e.Contains(u.UserCode)));
            var userInfo = await _userDBService.FindUserByNameAsync(request.AnyAccount);
            if (checkUser(userInfo, exceptUserCodes))
            {
                return (int)LoginProcessorTypeEnum.Official;
            }
            userInfo = await _userDBService.FindUserByPhoneAsync(request.AnyAccount);
            if (checkUser(userInfo, exceptUserCodes))
            {
                return (int)LoginProcessorTypeEnum.Phone;
            }
            userInfo = await _userDBService.FindUserByEmailAsync(request.AnyAccount);
            if (checkUser(userInfo, exceptUserCodes))
            {
                return (int)LoginProcessorTypeEnum.Email;
            }
            return (int)LoginProcessorTypeEnum.Unregistered;
        }

        private async Task<string> GetClientIdByTokenAsync(string token)
        {
            var request = new TokenRequest() { Token = token };
            var result = await _authenticationService.GetTokenAsync(request);
            return result.ClientId; //return User code 
        }


        public async Task<bool> ModifyPasswordAsync(ModifyPasswordRequest request)
        {
            VerifyAccountValue(request.AnyAccount);
            var userCode = await GetClientIdByTokenAsync(request.Token);
            var userInfo = await _userDBService.FindUserPasswordByCodeAsync(userCode);
            if (userInfo == null)
            {
                ThrowCustomerException(CustomerRpcCode.UserNotExisted, "User not found");
                return false;
            }
            var modifyType = LoginProcessorTypeEnum.Official;
            if (!string.IsNullOrWhiteSpace(request.AnyCode))
            {
                if (Regex.IsMatch(request.AnyAccount, "^1[0-9]{10}$"))
                {
                    modifyType = LoginProcessorTypeEnum.Phone;
                }
                else
                {
                    modifyType = LoginProcessorTypeEnum.Email;
                }
            }
            switch (modifyType)
            {
                case LoginProcessorTypeEnum.Phone:
                    if (request.AnyAccount != userInfo.Phone)
                    {
                        ThrowCustomerException(CustomerRpcCode.PhoneIllegality, "Phone is illegality");
                        return false;
                    }
                    var checkPhoneRight = await _smsService.CheckVerificationCodeAsync(request.AnyAccount, request.AnyCode);
                    if (!checkPhoneRight)
                    {
                        ThrowCustomerException(CustomerRpcCode.VerificationCodeIllegality, "VerificationCode is illegality");
                        return false;
                    }
                    break;
                case LoginProcessorTypeEnum.Email:
                    if (request.AnyAccount != userInfo.Email)
                    {
                        ThrowCustomerException(CustomerRpcCode.EmailIllegality, "Email is illegality");
                        return false;
                    }
                    var checkEmailRight = await CheckEmailVerificationCodeAsync(new CheckEmailVerificationCodeRequest { EmailAddress = request.AnyAccount.ToString(), VerifyCode = request.AnyCode });
                    if (!checkEmailRight)
                    {
                        ThrowCustomerException(CustomerRpcCode.VerificationCodeIllegality, "VerificationCode is illegality");
                        return false;
                    }
                    break;
                case LoginProcessorTypeEnum.Official:
                    if (request.AnyAccount != userInfo.UserName)
                    {
                        ThrowCustomerException(CustomerRpcCode.UserNameIllegality, "UserName is illegality");
                        return false;
                    }
                    if (!await _userDBService.VerificationPasswordAsync(userInfo.UserCode, request.Password))
                    {
                        ThrowCustomerException(CustomerRpcCode.OldPasswordError, "Old password is wrong!");
                        return false;
                    }
                    break;
                default:
                    ThrowCustomerException(CustomerRpcCode.UserNotRegistered, "User not registered");
                    return false;
            }
            //修改密码
            //if (string.IsNullOrWhiteSpace(request.NewPassword) || !Regex.IsMatch(request.NewPassword, @"^(?=.*[A-Za-z])(?=.*\d)[A-Za-z\d]{6,12}$"))
            if (string.IsNullOrWhiteSpace(request.NewPassword))
            {
                ThrowCustomerException(CustomerRpcCode.NewPasswordEmpty, "Incorrect NewPasswrod format");
                return false;
            }
            var _isResearchEditionService = EnvironmentConfigManager.GetParammeter<BoolParameter>("General", "IsResearchEditionService").Value;
            if (!_isResearchEditionService)
            {
                //验证密码是否重复
                var pwdVerifyReq = new PasswordVerifyModel()
                {
                    PasswordRecords = userInfo.PasswordRecords,
                    NewPassword = request.NewPassword
                };
                var boolean = _passwordExpireVerify.VerifySamePassword(pwdVerifyReq);
                if (boolean)
                {
                    if (_keepPwdTimes <= 0)
                    {
                        _keepPwdTimes = 5;
                    }
                    //没有包含，则添加
                    if (userInfo.PasswordRecords.Count >= _keepPwdTimes)
                    {
                        userInfo.PasswordRecords.RemoveAt(0);
                    }
                    if (userInfo.PasswordRecords.Count == 0)
                    {
                        userInfo.PasswordRecords.Add(userInfo.SecretPassword);
                    }
                    userInfo.PasswordRecords.Add(request.NewPassword);
                    userInfo.PasswordUpdateTime = DateTime.UtcNow;
                }
                else
                {
                    //验证不通过，
                    ThrowCustomerException(CustomerRpcCode.UserPasswordRepeatError, "User password repeat error");
                }
            }
            CacheMaintenance.Instance.Get<IUserInfoManager>().Remove(userInfo.UserCode);
            var result = await _userDBService.UpdateUserPasswordAsync(userInfo.UserCode, request.NewPassword, userInfo.PasswordRecords, userInfo.PasswordUpdateTime);
            if (result)
            {
                // //移除websocket
                // var tokenRequest = new TokenRequest { Token = request.Token };
                // var removeWebSocketResult = await _notificationService.RemoveToken(tokenRequest);
            }
            return result;
        }

        public async Task<string> GenerateNewPasswordAsync(GenerateNewPasswordRequest request)
        {
            var randomStr = GenerateRandomStr();
            return await Task.FromResult(randomStr);
        }

        private string GenerateRandomStr()
        {
            char[] constant1 =
            {
                'a','b','c','d','e','f','g','h','i','j','k','l','m','n','o','p','q','r','s','t','u','v','w','x','y','z',
                'A','B','C','D','E','F','G','H','I','J','K','L','M','N','O','P','Q','R','S','T','U','V','W','X','Y','Z'
            };
            var newRandom1 = new System.Text.StringBuilder(52);
            Random rd = new Random();
            for (int i = 0; i < 6; i++)
            {
                newRandom1.Append(constant1[rd.Next(52)]);
            }
            char[] constant2 =
            {
                '0','1','2','3','4','5','6','7','8','9'
            };
            var newRandom2 = new System.Text.StringBuilder(10);
            for (int i = 0; i < 3; i++)
            {
                newRandom2.Append(constant2[rd.Next(10)]);
            }
            char[] constant3 =
            {
                '!','@','#','$','%','^','&','*','(',')','_','+'
            };
            var newRandom3 = new System.Text.StringBuilder(12);
            for (int i = 0; i < 3; i++)
            {
                newRandom3.Append(constant3[rd.Next(12)]);
            }
            return $"{newRandom1.ToString()}{newRandom2.ToString()}{newRandom3.ToString()}";
        }

        /// <summary>
        /// 发起关联账户
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<StartAssociatedWithAccountResult> StartAssociatedWithAccountAsync(StartAssociatedWithAccountRequest request)
        {
            Logger.WriteLineInfo($"StartAssociatedWithAccountAsync:{JsonConvert.SerializeObject(request)}");
            UserDTO user = null;
            var result = new StartAssociatedWithAccountResult();
            if (request.IsDemission)
            {
                List<UserDTO> users = new List<UserDTO>();
                if (!string.IsNullOrWhiteSpace(request.ThirdPartyUserId))
                {
                    var userInfo = await _userDBService.FindUserByCommonPlatformUserIdAsync(request.ThirdPartyUserId);
                    if (userInfo != null)
                    {
                        users.Add(userInfo);
                    }
                }
                else
                {
                    if (!string.IsNullOrWhiteSpace(request.WingUserName))
                    {
                        var userInfo = await _userDBService.FindUserByNameAsync(request.WingUserName);
                        if (userInfo != null)
                        {
                            users.Add(userInfo);
                        }
                    }
                    if (!string.IsNullOrWhiteSpace(request.WingUserName))
                    {
                        var userInfo = await _userDBService.FindUserByPhoneAsync(request.WingUserName);
                        if (userInfo != null)
                        {
                            if (!users.Any(a => a.UserCode == userInfo.UserCode))
                            {
                                users.Add(userInfo);
                            }
                        }
                    }
                    if (!string.IsNullOrWhiteSpace(request.WingUserName))
                    {
                        var userInfo = await _userDBService.FindUserByEmailAsync(request.WingUserName);
                        if (userInfo != null)
                        {
                            if (!users.Any(a => a.UserCode == userInfo.UserCode))
                            {
                                users.Add(userInfo);
                            }
                        }
                    }
                    if (!string.IsNullOrWhiteSpace(request.Phone))
                    {
                        var userInfo = await _userDBService.FindUserByNameAsync(request.Phone);
                        if (userInfo != null)
                        {
                            if (!users.Any(a => a.UserCode == userInfo.UserCode))
                            {
                                users.Add(userInfo);
                            }
                        }
                    }
                    if (!string.IsNullOrWhiteSpace(request.Phone))
                    {
                        var userInfo = await _userDBService.FindUserByPhoneAsync(request.Phone);
                        if (userInfo != null)
                        {
                            if (!users.Any(a => a.UserCode == userInfo.UserCode))
                            {
                                users.Add(userInfo);
                            }
                        }
                    }
                    if (!string.IsNullOrWhiteSpace(request.Email))
                    {
                        var userInfo = await _userDBService.FindUserByNameAsync(request.Email);
                        if (userInfo != null)
                        {
                            if (!users.Any(a => a.UserCode == userInfo.UserCode))
                            {
                                users.Add(userInfo);
                            }
                        }
                    }
                    if (!string.IsNullOrWhiteSpace(request.Email))
                    {
                        var userInfo = await _userDBService.FindUserByEmailAsync(request.Email);
                        if (userInfo != null)
                        {
                            if (!users.Any(a => a.UserCode == userInfo.UserCode))
                            {
                                users.Add(userInfo);
                            }
                        }
                    }
                }
                if (users.Count > 0)
                {
                    //调用注销接口
                    foreach (var userData in users)
                    {
                        var userPasswordDTO = userData.MappingTo<UserPasswordDTO>();
                        //需要先将登录状态移除
                        var currentTokens = await _authenticationService.GetTokensWithClientIdAsync(new GetTokensWithClientIdRequest
                        {
                            ClientId = userData.UserCode,
                            AccountType = AccountType.User
                        });
                        foreach (var tokenInfo in currentTokens)
                        {
                            var loginOutResult = await _authenticationService.LogOffAsync(new TokenRequest
                            {
                                Token = tokenInfo.Code
                            });
                            if (loginOutResult)
                            {
                                //移除websocket
                                await _notificationService.RemoveToken(request);
                            }
                        }
                        //清理User关联信息
                        //同步注销绑定的管理员
                        var res = await _userDBService.CommonUserLogOffDBAsync(userPasswordDTO);
                        if (res)
                        {
                            result.IsSuccess = true;
                            result.State = AssociatedAccountResultEnum.Success;
                        }
                    }
                    return result;
                }
            }
            if (!string.IsNullOrWhiteSpace(request.WingUserName))
            {
                user = await _userDBService.FindUserByNameAsync(request.WingUserName);
            }
            if (user == null)
            {
                if (!string.IsNullOrEmpty(request.ThirdPartyUserId) && string.IsNullOrEmpty(request.WingUserName))
                {
                    //清空权限
                    if (await _userDBService.ClearAssociatedInfosAsync(new WingInterfaceLibrary.Request.DBRequest.ClearAssociatedInfosDBRequest
                    {
                        CommonPlatformUserId = request.ThirdPartyUserId,
                    }))
                    {
                        result.IsSuccess = true;
                        result.State = AssociatedAccountResultEnum.Success;
                        return result;
                    }
                }
                else
                {
                    result.State = AssociatedAccountResultEnum.UserNotFind;
                    return result;
                }
            }
            if (request.NeedPullData)
            {
                var auth = GetAuth();
                var reqStr = $"{{\"Auth\": \"{auth}\",\"XlhId\": \"{user.UserCode}\",\"UserId\": \"{request.ThirdPartyUserId}\",\"SystemCode\": \"Flyinsono\",\"Language\": \"{request.LanguageCode}\"}}";
                Logger.WriteLineInfo($"GetPermissionsAsync request:{reqStr}");
                var resStr = await SendRequest(_thirdPartyServer, "GetPermissionsAsync", reqStr);
                if (!string.IsNullOrWhiteSpace(resStr))
                {
                    Logger.WriteLineInfo($"GetPermissionsAsync result:{resStr}");
                    dynamic rpcRes = JObject.Parse(resStr);
                    dynamic data = rpcRes?.result?.Data?.List;
                    List<AssociatedInfoDTO> dtoList = new();
                    var list = (JArray)data;
                    if (list != null && list.Count > 0)
                    {
                        foreach (dynamic associatedInfo in list)
                        {
                            if (associatedInfo != null)
                            {
                                var dto = new AssociatedInfoDTO();
                                dto.Id = (string)associatedInfo.Id;
                                dto.RelationName = request.WingUserName;
                                dto.Title = (string)associatedInfo.Title;
                                dto.CTitle = (string)associatedInfo.CTitle;
                                dto.ETitle = (string)associatedInfo.ETitle;
                                dto.Icon = _curCDNHost + "/EA34BFF115404DE1BDA789001ECC1249.jpg";
                                var generlIconUrl = "";
                                if (dto.ETitle == "Feedback System")
                                {
                                    generlIconUrl = EnvironmentConfigManager.GetParammeter<StringParameter>("General", "AssociatedSystemIcon").Value;
                                }
                                else if (dto.ETitle.Contains("AgentAssessment") || request.AssociateType == AssociateTypeEnum.AgentAssessmentSystem)
                                {
                                    generlIconUrl = EnvironmentConfigManager.GetParammeter<StringParameter>("General", "AgentAssessmentSystemIcon").Value;
                                }
                                else
                                {
                                    dto.Icon = "";
                                }
                                if (!string.IsNullOrEmpty(generlIconUrl))
                                {
                                    var fileName = Path.GetFileName(generlIconUrl);
                                    dto.Icon = _curCDNHost + "/" + fileName;
                                }
                                dto.Description = (string)associatedInfo.Description;
                                dto.Url = (string)associatedInfo.Url;
                                dto.Index = (int)associatedInfo.Index;
                                dtoList.Add(dto);
                            }
                        }
                    }
                    if (await _userDBService.UpdateAssociatedInfosAsync(new WingInterfaceLibrary.Request.DBRequest.UpdateAssociatedInfosDBRequest
                    {
                        Code = user.UserCode,
                        CommonPlatformUserId = string.IsNullOrEmpty(request.ThirdPartyUserId) ? "" : request.ThirdPartyUserId,
                        AssociatedInfos = dtoList
                    }))
                    {
                        result.IsSuccess = true;
                        result.State = AssociatedAccountResultEnum.Success;
                        return result;
                    }
                }
            }
            else
            {
                result.IsSuccess = true;
                result.State = AssociatedAccountResultEnum.Success;
                return result;
            }
            return result;
        }

        // /// <summary>
        // /// 代理商发起关联账户
        // /// </summary>
        // /// <param name="request"></param>
        // /// <returns></returns>
        // private async Task<bool> SendUserAgentInfoXlhAsync(UserDTO userInfo)
        // {
        //     var res = false;
        //     try
        //     {
        //         var userPasswordDTO = userInfo.MappingTo<UserPasswordDTO>();
        //         if (!userPasswordDTO.IsOldAgent)
        //         {
        //             userPasswordDTO.IsOldAgent = true;
        //             CacheMaintenance.Instance.Get<IUserInfoManager>().Remove(userPasswordDTO.UserCode);
        //             await _userDBService.UpdateUserAsync(userPasswordDTO.UserCode, userPasswordDTO, string.Empty, true);
        //         }
        //         var objInfo = new
        //         {
        //             Token = _agentToken,
        //             SystemCode = "AgentAssessment",
        //             Language = 0,
        //             IsHandel = 0,
        //             Id = userInfo.UserCode,
        //             UserName = userInfo.UserName,
        //             FullName = userInfo.FullName,
        //             PhoneNumber = userInfo.Phone,
        //             Email = userInfo.Email
        //         };
        //         var json = JsonConvert.SerializeObject(objInfo);
        //         var reqStr = $"{json}";
        //         var result = await SendRequest(_agentServer, "SyncUserInfoXlhAsync", reqStr);
        //         if (!string.IsNullOrWhiteSpace(result))
        //         {
        //             JObject resultObj = JObject.Parse(result);
        //             var returnCode = resultObj["result"]["Code"].ToString();
        //             if (returnCode == "0")
        //             {
        //                 res = true;
        //             }
        //         }
        //     }
        //     catch (Exception ex)
        //     {
        //         Logger.WriteLineError($"Send UserAgentInfoXlhAsync Infos Error, ex:{ex}");
        //     }
        //     return res;
        // }

        // /// <summary>
        // /// 发起关联账户
        // /// </summary>
        // /// <param name="request"></param>
        // /// <returns></returns>
        // public async Task<ClearAssociatedWithAccountResult> ClearAssociatedWithAccountAsync(ClearAssociatedWithAccountRequest request)
        // {
        //     Logger.WriteLineInfo($"ClearAssociatedWithAccountAsync:{JsonConvert.SerializeObject(request)}");
        //     var result = new ClearAssociatedWithAccountResult();
        //     if (!string.IsNullOrWhiteSpace(request.ThirdPartyUserId))
        //     {
        //         if (await _userDBService.ClearAssociatedInfosAsync(new WingInterfaceLibrary.Request.DBRequest.ClearAssociatedInfosDBRequest
        //         {
        //             CommonPlatformUserId = request.ThirdPartyUserId,
        //         }))
        //         {
        //             result.IsSuccess = true;
        //             result.State = AssociatedAccountResultEnum.Success;
        //             return result;
        //         }
        //     }
        //     return result;
        // }

        /// <summary>
        /// 获取关联账户信息
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<GetAssociatedAccountInfoResult> GetAssociatedAccountInfoAsync(GetAssociatedAccountInfoRequest request)
        {
            Logger.WriteLineInfo($"GetAssociatedAccountInfoAsync:{JsonConvert.SerializeObject(request)}");
            GetAssociatedAccountInfoResult result = new();
            var userCode = await GetClientIdByTokenAsync(request.Token);
            if (!string.IsNullOrWhiteSpace(userCode))
            {
                var userInfo = await _userDBService.FindUserByCodeAsync(userCode);
                if (userInfo != null)
                {
                    result.AccountInfoList = new List<AssociatedFeatureInfoDTO>();
                    if (request.AssociateType == null || request.AssociateType == AssociateTypeEnum.All)
                    {
                        foreach (var associatedInfo in userInfo.AssociatedInfos)
                        {
                            if (associatedInfo.ETitle == "Feedback System")
                            {
                                associatedInfo.Icon = EnvironmentConfigManager.GetParammeter<StringParameter>("General", "AssociatedSystemIcon").Value ?? "MainMenu";
                            }
                            else if (associatedInfo.ETitle == "AgentAssessment")
                            {
                                associatedInfo.Icon = EnvironmentConfigManager.GetParammeter<StringParameter>("General", "AgentAssessmentSystemIcon").Value ?? "PersonCenter";
                            }
                            var relationName = string.IsNullOrWhiteSpace(associatedInfo.RelationName) ? userInfo.UserName : associatedInfo.RelationName;
                            var systemCode = associatedInfo.ETitle.Replace(" ", "");
                            var newUrl = "";//_curCDNHost + "/EA34BFF115404DE1BDA789001ECC1249.jpg";
                            if (!string.IsNullOrEmpty(associatedInfo.Icon))
                            {
                                var fileName = Path.GetFileName(associatedInfo.Icon);
                                newUrl = _curCDNHost + "/" + fileName;
                            }
                            var model = new AssociatedFeatureInfoDTO
                            {
                                FeatureCode = systemCode,
                                Id = associatedInfo.Id,
                                Title = associatedInfo.Title,
                                CTitle = associatedInfo.CTitle,
                                ETitle = associatedInfo.ETitle,
                                Icon = newUrl,
                                Description = associatedInfo.Description,
                                Url = GetAssociatedUrl(systemCode, request.Token, relationName, associatedInfo.Url),
                                Index = associatedInfo.Index
                            };
                            if (model.ETitle == "Feedback System")
                            {
                                model.AssociatedPosition = EnvironmentConfigManager.GetParammeter<StringParameter>("General", "AssociatedSystemPosition").Value ?? "MainMenu";
                            }
                            else if (model.ETitle == "AgentAssessment")
                            {
                                model.AssociatedPosition = EnvironmentConfigManager.GetParammeter<StringParameter>("General", "AgentAssessmentSystemPosition").Value ?? "PersonCenter";
                                model.CTitle = model.CTitle.Replace("代理商服务系统", "代理商服务");
                                model.ETitle = model.ETitle.Replace("AgentAssessment", "Agent Service");
                            }
                            result.AccountInfoList.Add(model);
                            result.IsSuccess = true;
                        }
                    }
                    else
                    {
                        if (userInfo.AssociatedInfos?.Count > 0)
                        {
                            var associatedInfo = new AssociatedInfoDTO();
                            if (request.AssociateType == AssociateTypeEnum.FeedbackSystem)
                            {
                                associatedInfo = userInfo.AssociatedInfos.FirstOrDefault(c => c.ETitle.Equals("Feedback System"));
                            }
                            else if (request.AssociateType == AssociateTypeEnum.AgentAssessmentSystem)
                            {
                                associatedInfo = userInfo.AssociatedInfos.FirstOrDefault(c => c.ETitle.Equals("AgentAssessment"));
                            }
                            if (associatedInfo != null)
                            {
                                if (request.AssociateType == AssociateTypeEnum.FeedbackSystem)
                                {
                                    associatedInfo.Icon = EnvironmentConfigManager.GetParammeter<StringParameter>("General", "AssociatedSystemIcon").Value;
                                }
                                else if (request.AssociateType == AssociateTypeEnum.AgentAssessmentSystem)
                                {
                                    associatedInfo.Icon = EnvironmentConfigManager.GetParammeter<StringParameter>("General", "AgentAssessmentSystemIcon").Value;
                                }
                                var relationName = string.IsNullOrWhiteSpace(associatedInfo.RelationName) ? userInfo.UserName : associatedInfo.RelationName;
                                var systemCode = associatedInfo.ETitle.Replace(" ", "");
                                var newUrl = "";//_curCDNHost + "/EA34BFF115404DE1BDA789001ECC1249.jpg";
                                if (!string.IsNullOrEmpty(associatedInfo.Icon))
                                {
                                    var fileName = Path.GetFileName(associatedInfo.Icon);
                                    newUrl = _curCDNHost + "/" + fileName;
                                }
                                var respEntity = new AssociatedFeatureInfoDTO
                                {
                                    FeatureCode = systemCode,
                                    Id = associatedInfo.Id,
                                    Title = associatedInfo.Title,
                                    CTitle = associatedInfo.CTitle,
                                    ETitle = associatedInfo.ETitle,
                                    Icon = newUrl,
                                    Description = associatedInfo.Description,
                                    Url = GetAssociatedUrl(systemCode, request.Token, relationName, associatedInfo.Url),
                                    Index = associatedInfo.Index
                                };
                                if (request.AssociateType == AssociateTypeEnum.FeedbackSystem || respEntity.ETitle == "Feedback System")
                                {
                                    respEntity.AssociatedPosition = EnvironmentConfigManager.GetParammeter<StringParameter>("General", "AssociatedSystemPosition").Value ?? "MainMenu";
                                }
                                else if (request.AssociateType == AssociateTypeEnum.AgentAssessmentSystem || respEntity.ETitle == "AgentAssessment")
                                {
                                    respEntity.AssociatedPosition = EnvironmentConfigManager.GetParammeter<StringParameter>("General", "AgentAssessmentSystemPosition").Value ?? "PersonCenter";
                                    respEntity.CTitle = respEntity.CTitle.Replace("代理商服务系统", "代理商服务");
                                    respEntity.ETitle = respEntity.ETitle.Replace("AgentAssessment", "Agent Service");
                                }
                                result.AccountInfoList.Add(respEntity);
                                result.IsSuccess = true;
                            }
                        }
                    }
                }
            }
            return result;
        }

        /// <summary>
        /// 接收到同步信息
        /// </summary>
        /// <param name="request">The request</param>
        /// <returns></returns>
        /// <show>false</show>
        public async Task<bool> ReceiveSyncServerMessageAsync(SyncReceiveServiceDataRequest request)
        {
            try
            {
                Logger.WriteLineInfo($"LoginService SyncServerMessageAsync, source url:{request.SourceUrl}, syncType:{request.SyncType.ToString()}");
                var userScanInfo = JsonConvert.DeserializeObject<UserScanInfo>(request.ServiceDataJson);
                var result = await _scanLoginManager.AddOrUpdate(userScanInfo, 1);
                return result;
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"NotificationService SyncServerMessageAsync err, {ex}");
                return false;
            }
        }

        /// <summary>
        /// 获取扫码唯一码接口
        /// </summary>
        /// <returns></returns>
        public async Task<GetScanCodeResult> GetScanCodeAsync(GetScanCodeRequest request)
        {
            var result = new GetScanCodeResult();
            var scanLoginSource = request.ScanLoginSource;
            var installVersion = request.InstallVersion;
            var scanCode = Guid.NewGuid().ToString();
            var platform = request.Platform;
            var validSeconds = _scanLoginValidSeconds;
            var userScanInfo = new UserScanInfo
            {
                ScanCode = scanCode,
                CreateTime = DateTime.UtcNow,
                ExpireTime = DateTime.UtcNow.AddSeconds(validSeconds + 10),//超时时间
                ScanLoginSource = scanLoginSource,
                InstallVersion = installVersion,
                Platform = platform
            };
            await _scanLoginManager.AddOrUpdate(userScanInfo);
            result.ScanCode = scanCode;
            result.ValidSeconds = validSeconds;
            return result;
        }

        /// <summary>
        /// 确认扫码
        /// </summary>
        /// <returns></returns>
        public async Task<bool> ConfirmScanAsync(ConfirmScanRequest request)
        {
            var scanCode = request.ScanCode;
            var userScanInfo = _scanLoginManager.Get(scanCode);
            if (userScanInfo == null || userScanInfo.ExpireTime < DateTime.UtcNow)
            {
                ThrowCustomerException(CustomerRpcCode.UserScanInfoError, "UserScanInfoError");
            }
            var userCode = await GetClientIdByTokenAsync(request.Token);
            var userInfo = await _userDBService.FindUserByCodeAsync(userCode);
            //根据LoginSource生成token并返回
            var platform = userScanInfo.Platform;
            var loginSource = GetLoginSource(userScanInfo.ScanLoginSource);
            var loginServer = ConfigurationManager.Host;
            var ipValue = 0;//should be from userLoginInfo
            var installVersion = userScanInfo.InstallVersion;
            var applyRequest = new ApplyTokenRequest(AccountType.User, userInfo.UserName, platform, loginSource, userCode, loginServer, ipValue, installVersion);
            var tokenInfo = await _authenticationService.ApplyTokenAsync(applyRequest);
            userScanInfo.LoginResult = new LoginResult { LoginState = LoginStateEnum.Succeed, Token = tokenInfo.Code, AccountName = userInfo.UserName };
            userScanInfo.ExpireTime = userScanInfo.ExpireTime.AddMinutes(2);
            await _scanLoginManager.AddOrUpdate(userScanInfo);
            return true;
        }

        /// <summary>
        /// 查询扫码确认状态
        /// </summary>
        /// <returns></returns>
        public async Task<CheckConfirmScanStateResult> CheckConfirmScanStateAsync(CheckConfirmScanStateRequest request)
        {
            var result = new CheckConfirmScanStateResult();
            var state = CheckConfirmScanState.Wait;
            var scanCode = request.ScanCode;
            var userScanInfo = _scanLoginManager.Get(scanCode);
            if (userScanInfo == null || userScanInfo.ExpireTime < DateTime.UtcNow)
            {
                //可两次失败
                if (ScanLoginManager.ExpireOrErrorScanCodes.Contains(scanCode))
                {
                    state = CheckConfirmScanState.Expire;
                }
                else
                {
                    ScanLoginManager.ExpireOrErrorScanCodes.Add(scanCode);
                }
            }
            else if (userScanInfo.LoginResult == null || string.IsNullOrWhiteSpace(userScanInfo.LoginResult.Token))
            {
                state = CheckConfirmScanState.Wait;
            }
            else
            {
                state = CheckConfirmScanState.Success;
                result.LoginResult = userScanInfo.LoginResult;
            }
            result.CheckConfirmScanState = state;
            return result;
        }

        private string GetAssociatedUrl(string systemCode, string token, string userName, string url)
        {
            Logger.WriteLineInfo($"LoginService GetAssociatedUrl Request, systemCode:{systemCode}, token:{token}, userName:{userName}, url:{url}");
            switch (systemCode)
            {
                case "FeedbackSystem":
                    var tokenInfo = CacheMaintenance.Instance.Get<ITokensManager>().Where(x => x.Code == token).FirstOrDefault();
                    var loginSourceName = Enum.GetName<LoginSource>((LoginSource)tokenInfo.LoginSource);
                    byte[] hashedDataBytes = Encoding.GetEncoding("gb2312").GetBytes($"{DateTime.UtcNow.ToString("yyyy-MM-dd")}-{userName}-0003");
                    var associatedToken = WingServerCommon.Utilities.MD5.GetHashString(hashedDataBytes);
                    url = url.Replace("{type}", loginSourceName).Replace("{account}", userName).Replace("{token}", associatedToken);
                    break;
                case "AgentAssessment":
                    url = url.Replace("{userId}", userName).Replace("{account}", userName);
                    break;
            }
            return url;
        }

        private LoginSource GetLoginSource(ScanLoginSource scanLoginSource)
        {
            var loginSource = LoginSource.Web;
            switch (scanLoginSource)
            {
                case ScanLoginSource.PC:
                    loginSource = LoginSource.PC;
                    break;
                case ScanLoginSource.Web:
                    loginSource = LoginSource.Web;
                    break;
                case ScanLoginSource.US:
                    loginSource = LoginSource.US;
                    break;
                default:
                    break;
            }
            return loginSource;
        }

        private string GetAuth()
        {
            byte[] bytes = WingServerCommon.Utilities.MD5.GetHash(Encoding.UTF8.GetBytes($"{DateTime.Now.ToString("yyyyMMdd")}VinnoBasicplatform"));
            return Convert.ToBase64String(bytes);
        }

        /// <summary>
        /// 发送请求
        /// </summary>
        /// <param name="server">服务器</param>
        /// <param name="req">请求信息</param>
        /// <returns></returns>
        private async Task<string> SendRequest(string server, string method, string req)
        {
            var requestJson = @"{
                ""jsonrpc"": ""2.0"",
                ""method"": """ + method + @""",
                ""params"": [" + req + @"],
                ""id"": """ + DateTime.UtcNow.Ticks + @"""
            }";
            var request = new HttpRequestMessage(HttpMethod.Post, server);
            request.Content = new StringContent(requestJson);
            var result = _httpClient.Send(request);
            return await result.Content.ReadAsStringAsync();
        }
    }
}
