﻿using WingServerCommon.Service;
using System.Threading.Tasks;
using WingInterfaceLibrary.Result;
using WingInterfaceLibrary.DB.Request;
using WingInterfaceLibrary.DTO.Education;
using WingInterfaceLibrary.DTO.Record;
using WingInterfaceLibrary.DTO.User;
using System.Collections.Generic;
using System.Linq;
using WingServerCommon.Mapper;
using WingInterfaceLibrary.Request.User;
using WingInterfaceLibrary.DTO.Device;
using WingInterfaceLibrary.Interface;
using WingInterfaceLibrary.DTO.Organization;
using System;
using WingInterfaceLibrary.Enum;

namespace WingUserModule.Service;

public partial class UserService : JsonRpcService, IUserService
{
    /// <summary>
    /// 分页查询用户群组
    /// </summary>
    /// <param name="request"></param>
    /// <returns></returns>
    public async Task<PageResult<UserGroupDTO>> GetUserGroupPagesAsync(GetUserGroupPagesRequest request)
    {
        var userCode = await GetClientIdByTokenAsync(request.Token);
        var userDTO = await _userDBService.FindUserByCodeAsync(userCode);
        if (userDTO == null || userDTO.UserFeatureCodes == null || !userDTO.UserFeatureCodes.Contains("UserGroups"))
        {
            ThrowCustomerException(CustomerRpcCode.NoUserGroupsFeature, "No UserGroups Feature");
        }

        var keyword = request.Keyword;
        var pagedData = await _userGroupDBService.GetUserGroupByPageDBAsync(new GetUserGroupByPageDBRequest
        {
            PageIndex = request.PageIndex,
            PageSize = request.PageSize,
            Keyword = keyword,
            UserCode = userCode,
        });
        return pagedData;
    }

    /// <summary>
    /// 查询用户群组详情
    /// </summary>
    /// <param name="request"></param>
    /// <returns></returns>
    public async Task<UserGroupDTO> GetUserGroupAsync(GetUserGroupRequest request)
    {
        var userCode = await GetClientIdByTokenAsync(request.Token);
        var userDTO = await _userDBService.FindUserByCodeAsync(userCode);
        if (userDTO == null || userDTO.UserFeatureCodes == null || !userDTO.UserFeatureCodes.Contains("UserGroups"))
        {
            ThrowCustomerException(CustomerRpcCode.NoUserGroupsFeature, "No UserGroups Feature");
        }

        var userGroupCode = request.UserGroupCode;
        var dto = await _userGroupDBService.GetUserGroupDBAsync(new GetUserGroupDBRequest
        {
            UserGroupCode = userGroupCode,
        });
        return dto;
    }

    /// <summary>
    /// 查询群组内用户集合
    /// </summary>
    /// <param name="request"></param>
    /// <returns></returns>
    public async Task<PageResult<UserExtendDTO>> GetGroupUserInfosAsync(GetGroupUserInfosRequest request)
    {
        var languageCode = request.LanguageCode;
        var userGroupCode = request.UserGroupCode;
        var pageIndex = request.PageIndex;
        var pageSize = request.PageSize;
        var keyword = request.Keyword;

        var dto = await _userGroupDBService.GetUserGroupDBAsync(new GetUserGroupDBRequest
        {
            UserGroupCode = userGroupCode,
        });
        var userCodes = dto.UserCodes?.ToList();
        var users = await _userDBService.FindUserListByUserCodesAsync(userCodes);
        var resultData = users.MappingTo<List<UserExtendDTO>>();
        var findAuthenticationRolesRequest = new FindAuthenticationRolesRequest { Language = languageCode };
        var roles = await FindAllRolesAsync(findAuthenticationRolesRequest);
        foreach (var item in resultData)
        {
            if (item.RoleCodes != null && item.RoleCodes.Count > 0)
            {
                item.RoleName = roles.FirstOrDefault(x => x.RoleCode == item.RoleCodes[0])?.RoleName;
            }
        }
        if (!string.IsNullOrWhiteSpace(keyword))
        {
            resultData = resultData.Where(x => x.FullName?.Contains(keyword) == true || x.UserName?.Contains(keyword) == true || x.NickName?.Contains(keyword) == true
                || x.Phone?.Contains(keyword) == true || x.Email?.Contains(keyword) == true)?.ToList() ?? new List<UserExtendDTO>();
        }
        var totalCount = resultData.Count;
        resultData = resultData.OrderByDescending(o => o.CreateTime).Skip((pageIndex - 1) * pageSize).Take(pageSize).ToList();
        return new PageResult<UserExtendDTO>
        {
            PageIndex = pageIndex,
            PageSize = pageSize,
            TotalCount = totalCount,
            PageData = resultData,
        };
    }

    /// <summary>
    /// 查询群组内设备集合
    /// </summary>
    /// <param name="request"></param>
    /// <returns></returns>
    public async Task<PageResult<DeviceInfoDTO>> GetGroupDeviceInfosAsync(GetGroupDeviceInfosRequest request)
    {
        var userGroupCode = request.UserGroupCode;
        var pageIndex = request.PageIndex;
        var pageSize = request.PageSize;
        var keyword = request.Keyword;

        var dto = await _userGroupDBService.GetUserGroupDBAsync(new GetUserGroupDBRequest
        {
            UserGroupCode = userGroupCode,
        });
        var deviceCodes = dto.DeviceCodes?.ToList();
        var devices = await _deviceInfoDBService.FindDeviceInfoListByCodeListAsync(deviceCodes);
        if (!string.IsNullOrWhiteSpace(keyword))
        {
            devices = devices.Where(x => x.Description?.Contains(keyword) == true || x.Name?.Contains(keyword) == true
                || x.DeviceModel?.Contains(keyword) == true || x.ShortCode?.Contains(keyword) == true)?.ToList() ?? new List<DeviceInfoDTO>();
        }
        var totalCount = devices.Count;
        devices = devices.OrderByDescending(o => o.CreateTime).Skip((pageIndex - 1) * pageSize).Take(pageSize).ToList();
        return new PageResult<DeviceInfoDTO>
        {
            PageIndex = pageIndex,
            PageSize = pageSize,
            TotalCount = totalCount,
            PageData = devices,
        };
    }

    /// <summary>
    /// 查询群组内医院集合
    /// </summary>
    /// <param name="request"></param>
    /// <returns></returns>
    public async Task<PageResult<OrganizationSimplifyDTO>> GetGroupOrganizationsAsync(GetGroupOrganizationsRequest request)
    {
        var userGroupCode = request.UserGroupCode;
        var pageIndex = request.PageIndex;
        var pageSize = request.PageSize;
        var keyword = request.Keyword;

        var dto = await _userGroupDBService.GetUserGroupDBAsync(new GetUserGroupDBRequest
        {
            UserGroupCode = userGroupCode,
        });
        var deviceCodes = dto.DeviceCodes?.ToList();
        var devices = await _deviceInfoDBService.FindDeviceInfoListByCodeListAsync(deviceCodes);
        var organizationCodes = devices.Select(x => x.OrganizationCode).Distinct().ToList();
        var organizations = await _organizationDBService.FindOrganizationDTOListByCodesAsync(organizationCodes);
        if (!string.IsNullOrWhiteSpace(keyword))
        {
            organizations = organizations.Where(x => x.Description?.Contains(keyword) == true || x.OrganizationName?.Contains(keyword) == true)?.ToList() ?? new List<OrganizationDTO>();
        }
        var totalCount = organizations.Count;
        organizations = organizations.OrderByDescending(o => o.CreateTime).Skip((pageIndex - 1) * pageSize).Take(pageSize).ToList();
        var resultData = organizations.Select(x => new OrganizationSimplifyDTO
        {
            OrganizationCode = x.OrganizationCode,
            OrganizationName = x.OrganizationName,
            ShortCode = x.ShortCode,
        }).ToList();
        return new PageResult<OrganizationSimplifyDTO>
        {
            PageIndex = pageIndex,
            PageSize = pageSize,
            TotalCount = totalCount,
            PageData = resultData,
        };
    }

    /// <summary>
    /// 分页查询用户群组检查记录集合
    /// </summary>
    /// <param name="request"></param>
    /// <returns></returns>
    public async Task<PageResult<SimpleRecordInfoDTO>> GetUserGroupRecordPagesAsync(GetUserGroupRecordPagesRequest request)
    {
        var pageIndex = request.PageIndex;
        var pageSize = request.PageSize;
        var keyword = request.Keyword;
        var userGroupCode = request.UserGroupCode;
        var organizationCodes = request.OrganizationCodes;
        var deviceCodes = request.DeviceCodes;
        var startDate = request.StartDate;
        var endDate = request.EndDate;
        var userCode = await GetClientIdByTokenAsync(request.Token);
        var userDTO = await _userDBService.FindUserByCodeAsync(userCode);
        if (userDTO == null || userDTO.UserFeatureCodes == null || !userDTO.UserFeatureCodes.Contains("UserGroups"))
        {
            ThrowCustomerException(CustomerRpcCode.NoUserGroupsFeature, "No UserGroups Feature");
        }

        var userGroupDTO = await _userGroupDBService.GetUserGroupDBAsync(new GetUserGroupDBRequest { UserGroupCode = userGroupCode });
        if (userGroupDTO == null || string.IsNullOrWhiteSpace(userGroupDTO.Code) || userGroupDTO.DeviceCodes == null || !userGroupDTO.DeviceCodes.Any())
        {
            return new PageResult<SimpleRecordInfoDTO>
            {
                PageIndex = pageIndex,
                PageSize = pageSize,
                TotalCount = 0,
                PageData = new List<SimpleRecordInfoDTO>(),
            };
        }
        else
        {
            deviceCodes = userGroupDTO.DeviceCodes.ToList();
            var deviceDTOs = await _deviceInfoDBService.FindDeviceInfoListByCodeListAsync(deviceCodes);
            if (deviceDTOs != null && deviceDTOs.Any())
            {
                organizationCodes = deviceDTOs.Select(x => x.OrganizationCode).ToList();
            }
        }
        var dbRequest = new FindFullRecordInfoPagesRequest
        {
            Language = request.Language,
            UserCode = userCode,
            CurrentPage = request.PageIndex,
            PageSize = request.PageSize,
            OrganizationCodes = organizationCodes,
            DeviceCodes = deviceCodes,
            RecordProcessState = request.RecordProcessState,
            NotFilterByFeature = true
        };

        var filter = new Dictionary<string, string>();
        if (startDate > new DateTime(2000, 1, 1))
        {
            filter.Add("CreateTime>", startDate.Date.ToString("yyyy-MM-dd HH:mm:ssZ"));
        }
        if (endDate > new DateTime(2000, 1, 1))
        {
            filter.Add("CreateTime<", endDate.Date.AddDays(1).ToString("yyyy-MM-dd HH:mm:ssZ"));
        }
        if (!string.IsNullOrWhiteSpace(request.Keyword))
        {
            filter.Add("KeyWord", keyword);
        }

        dbRequest.Filter = filter;
        var result = await _recordsInfoDBService.FindRecordPagesAsync(dbRequest);
        return result;
    }
}
